@@ -1,16 +1,25 @@
 """
-Core models for active learning algorithms.
+==========================================
+Core models for active learning algorithms
+==========================================
 """
 
+import abc
+import sys
 import numpy as np
-from abc import ABC, abstractmethod
-from sklearn.utils import check_array
+
 from sklearn.base import BaseEstimator
 from modAL.utils.validation import check_class_labels, check_class_proba
 from modAL.uncertainty import uncertainty_sampling
 from modAL.disagreement import vote_entropy_sampling, max_std_sampling
 
 
+if sys.version_info >= (3, 4):
+    ABC = abc.ABC
+else:
+    ABC = abc.ABCMeta('ABC', (), {})
+
+
 class ActiveLearner(BaseEstimator):
     """
     This class is an abstract model of a general active learning algorithm.
@@ -101,8 +110,8 @@ class ActiveLearner(BaseEstimator):
             self.X_training = None
             self.y_training = None
         elif type(X_training) != type(None) and type(y_training) != type(None):
-            self.X_training = check_array(X_training)
-            self.y_training = check_array(y_training, ensure_2d=False)
+            self.X_training = X_training
+            self.y_training = y_training
             self._fit_to_known(bootstrap=bootstrap_init, **fit_kwargs)
 
     def _add_training_data(self, X, y):
@@ -125,7 +134,6 @@ class ActiveLearner(BaseEstimator):
         have to agree with the training samples which the
         classifier has seen.
         """
-        X, y = check_array(X), check_array(y, ensure_2d=False)
         assert len(X) == len(y), 'the number of new data points and number of labels must match'
 
         if type(self.X_training) != type(None):
@@ -162,6 +170,35 @@ class ActiveLearner(BaseEstimator):
             self.estimator.fit(self.X_training[bootstrap_idx], self.y_training[bootstrap_idx], **fit_kwargs)
             return self
 
+    def _fit_on_new(self, X, y, bootstrap=False, **fit_kwargs):
+        """
+        Fits self.estimator to the given data and labels.
+
+        Parameters
+        ----------
+        X: numpy.ndarray of shape (n_samples, n_features)
+            The new samples for which the labels are supplied
+            by the expert.
+
+        y: numpy.ndarray of shape (n_samples, )
+            Labels corresponding to the new instances in X.
+
+        bootstrap: boolean
+            If True, the method trains the model on a set bootstrapped from X.
+
+        fit_kwargs: keyword arguments
+            Keyword arguments to be passed to the fit method of the predictor.
+        """
+        assert len(X) == len(y), 'the length of X and y must match'
+
+        if not bootstrap:
+            self.estimator.fit(X, y, **fit_kwargs)
+            return self
+        else:
+            bootstrap_idx = np.random.choice(range(len(X)), len(X), replace=True)
+            self.estimator.fit(X[bootstrap_idx], y[bootstrap_idx])
+            return self
+
     def fit(self, X, y, bootstrap=False, **fit_kwargs):
         """
         Interface for the fit method of the predictor. Fits the predictor
@@ -257,7 +294,6 @@ class ActiveLearner(BaseEstimator):
         X[query_idx]: numpy.ndarray of shape (n_instances, n_features)
             The instances from X_pool chosen to be labelled.
         """
-        check_array(X, ensure_2d=True)
 
         query_idx, query_instances = self.query_strategy(self.estimator, X, **query_kwargs)
         return query_idx, query_instances
@@ -284,7 +320,7 @@ class ActiveLearner(BaseEstimator):
         """
         return self.estimator.score(X, y, **score_kwargs)
 
-    def teach(self, X, y, bootstrap=False, **fit_kwargs):
+    def teach(self, X, y, bootstrap=False, only_new=False, **fit_kwargs):
         """
         Adds X and y to the known training data and retrains the predictor
         with the augmented dataset.
@@ -302,15 +338,23 @@ class ActiveLearner(BaseEstimator):
             If True, training is done on a bootstrapped dataset. Useful for building
             Committee models with bagging.
 
+        only_new: boolean
+            If True, the model is retrained using only X and y, ignoring the previously
+            provided examples. Useful when working with models where the .fit() method
+            doesn't retrain the model from scratch. (For example, in tensorflow or keras.)
+
         fit_kwargs: keyword arguments
             Keyword arguments to be passed to the fit method
             of the predictor.
         """
         self._add_training_data(X, y)
-        self._fit_to_known(bootstrap=bootstrap, **fit_kwargs)
+        if not only_new:
+            self._fit_to_known(bootstrap=bootstrap, **fit_kwargs)
+        else:
+            self._fit_on_new(X, y, bootstrap=bootstrap, **fit_kwargs)
 
 
-class BaseCommittee(ABC):
+class BaseCommittee(ABC, BaseEstimator):
     def __init__(
             self,
             learner_list,                                        # list of ActiveLearner objects
@@ -368,6 +412,30 @@ class BaseCommittee(ABC):
         for learner in self.learner_list:
             learner._fit_to_known(bootstrap=bootstrap, **fit_kwargs)
 
+    def _fit_on_new(self, X, y, bootstrap=False, **fit_kwargs):
+        """
+        Fits all learners to the given data and labels.
+
+        Parameters
+        ----------
+        X: numpy.ndarray of shape (n_samples, n_features)
+            The new samples for which the labels are supplied
+            by the expert.
+
+        y: numpy.ndarray of shape (n_samples, )
+            Labels corresponding to the new instances in X.
+
+        bootstrap: boolean
+            If True, the method trains the model on a set bootstrapped from X.
+
+        fit_kwargs: keyword arguments
+            Keyword arguments to be passed to the fit method of the predictor.
+        """
+        assert len(X) == len(y), 'the length of X and y must match'
+
+        for learner in self.learner_list:
+            learner._fit_on_new(X, y, bootstrap=bootstrap, **fit_kwargs)
+
     def fit(self, X, y, **fit_kwargs):
         """
         Fits every learner to a subset sampled with replacement from X.
@@ -416,8 +484,6 @@ class BaseCommittee(ABC):
         X[query_idx]: numpy.ndarray of shape (n_instances, n_features)
             The instances from X_pool chosen to be labelled.
         """
-        check_array(X, ensure_2d=True)
-
         query_idx, query_instances = self.query_strategy(self, X, **query_kwargs)
         return query_idx, X[query_idx]
 
@@ -433,7 +499,7 @@ class BaseCommittee(ABC):
         """
         self._fit_to_known(bootstrap=True, **fit_kwargs)
 
-    def teach(self, X, y, bootstrap=False, **fit_kwargs):
+    def teach(self, X, y, bootstrap=False, only_new=False, **fit_kwargs):
         """
         Adds X and y to the known training data for each learner
         and retrains learners with the augmented dataset.
@@ -456,18 +522,21 @@ class BaseCommittee(ABC):
             of the predictor.
         """
         self._add_training_data(X, y)
-        self._fit_to_known(bootstrap=bootstrap, **fit_kwargs)
+        if not only_new:
+            self._fit_to_known(bootstrap=bootstrap, **fit_kwargs)
+        else:
+            self._fit_on_new(X, y, bootstrap=bootstrap, **fit_kwargs)
 
-    @abstractmethod
+    @abc.abstractmethod
     def predict(self, X):
         pass
 
-    @abstractmethod
+    @abc.abstractmethod
     def vote(self, X):
         pass
 
 
-class Committee(BaseCommittee, BaseEstimator):
+class Committee(BaseCommittee):
     """
     This class is an abstract model of a committee-based active learning algorithm.
 
@@ -616,7 +685,6 @@ class Committee(BaseCommittee, BaseEstimator):
             The predicted class for each learner in the Committee
             and each sample in X.
         """
-        check_array(X, ensure_2d=True)
         prediction = np.zeros(shape=(X.shape[0], len(self.learner_list)))
 
         for learner_idx, learner in enumerate(self.learner_list):
@@ -643,8 +711,6 @@ class Committee(BaseCommittee, BaseEstimator):
 
         """
 
-        check_array(X, ensure_2d=True)
-
         # get dimensions
         n_samples = X.shape[0]
         n_learners = len(self.learner_list)
@@ -669,7 +735,7 @@ class Committee(BaseCommittee, BaseEstimator):
         return proba
 
 
-class CommitteeRegressor(BaseCommittee, BaseEstimator):
+class CommitteeRegressor(BaseCommittee):
     """
     This class is an abstract model of a committee-based active learning regression.
 
@@ -772,7 +838,6 @@ class CommitteeRegressor(BaseCommittee, BaseEstimator):
         vote: numpy.ndarray of shape (n_samples, n_regressors)
             The predicted value for each regressor in the CommitteeRegressor and each sample in X.
         """
-        check_array(X, ensure_2d=True)
         prediction = np.zeros(shape=(len(X), len(self.learner_list)))
 
         for learner_idx, learner in enumerate(self.learner_list):
